package cn.fw.hermes.common.utils.im;

import cn.fw.common.exception.BusinessException;
import cn.fw.hermes.common.constant.CloudDataEnum;
import cn.fw.hermes.common.constant.ErrorCodeEnum;
import cn.fw.hermes.common.constant.MsgBody;
import cn.fw.hermes.common.utils.HttpClientPoolUtil;
import cn.fw.hermes.common.utils.HttpClientUtils;
import cn.fw.hermes.common.utils.StringUtils;
import cn.fw.hermes.sdk.api.para.MsgTypeEnum;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * IM发送工具类
 *
 * @author Kurisu
 */
@Slf4j
public class TXCloudUtils {

    /**
     * 账号绑定
     *
     * @param identifier      账号
     * @param nickname        昵称
     * @param image           头像
     * @param adminSig        管理员签名
     * @param adminIdentifier 管理员账号
     * @param appId           应用id
     * @return
     */
    public static JSONObject AccountImport(String identifier, String nickname, String image, String adminSig, String adminIdentifier, Long appId) {
        Map<String, String> map = new HashMap<>();
        map.put("Identifier", identifier);
        if (StringUtils.isValid(nickname)) {
            map.put("Nick", nickname);
        }
        if (StringUtils.isValid(image)) {
            map.put("FaceUrl", image);
        }
        //拼装json数据
        String json = JSON.toJSONString(map);
        //使用用户信息拼接url
        String url = TXCloudHelper.getUrl(CloudDataEnum.ACCOUNT_IMPORT.getCode(), adminSig, adminIdentifier, appId);
        //向腾讯接口发送请求并接受返回结果
        String result = HttpClientUtils.doPostJson(url, json);
        log.info("-------" + result);
        return JSON.parseObject(result);
    }

    /**
     * 获取用户资料接口
     *
     * @param adminSig
     * @param adminIdentifier
     * @param appId
     * @param tagList
     * @param list
     * @return
     */
    public static JSONObject getPortrait(String adminSig, String adminIdentifier, Long appId, List<String> tagList, List<String> list) {
        Map<String, List> map = new HashMap<>(2);
        Assert.notEmpty(tagList, "资料字段不能为空");
        Assert.notEmpty(list, "用户不能为空");
        map.put("To_Account", list);
        map.put("TagList", tagList);
        //拼装json数据
        String json = JSON.toJSONString(map);
        String url = TXCloudHelper.getUrl(CloudDataEnum.PORTRAIT_GET.getCode(), adminSig, adminIdentifier, appId);
        String result =  HttpClientPoolUtil.execute(url, 0, json);
        return JSON.parseObject(result);
    }

    /**
     * 设置用户信息
     *
     * @param adminSig
     * @param adminIdentifier
     * @param appId
     * @param tagMap
     * @param fromAccount
     * @return
     */
    public static JSONObject setPortrait(String adminSig, String adminIdentifier, Long appId, Map<String, Object> tagMap, String fromAccount) {
        Map<String, Object> map = new HashMap<>(2);
        map.put("From_Account", fromAccount);
        List<Map<String, Object>> list = new ArrayList<>();
        for (Map.Entry<String, Object> entry : tagMap.entrySet()) {
            Map<String, Object> stringObjectMap = new HashMap<>(2);
            stringObjectMap.put("Tag", entry.getKey());
            stringObjectMap.put("Value", entry.getValue());
            list.add(stringObjectMap);
        }
        map.put("ProfileItem", list);
        //拼装json数据
        String json = JSON.toJSONString(map);
        String url = TXCloudHelper.getUrl(CloudDataEnum.PORTRAIT_SET.getCode(), adminSig, adminIdentifier, appId);
        String result = HttpClientPoolUtil.execute(url, 0, json);
        return JSON.parseObject(result);
    }

    /**
     * 强制下线
     *
     * @param identifier      账号
     * @param adminSig        管理员签名
     * @param adminIdentifier 管理员账号
     * @param appId           应用id
     * @return
     */
    public static JSONObject Kick(String identifier, String adminSig, String adminIdentifier, Long appId) {
        Map<String, String> map = new HashMap<>(1);
        map.put("Identifier", identifier);
        //拼装json数据
        String json = JSON.toJSONString(map);
        String url = TXCloudHelper.getUrl(CloudDataEnum.KICK.getCode(), adminSig, adminIdentifier, appId);
        String result =  HttpClientPoolUtil.execute(url, 0, json);
        return JSON.parseObject(result);
    }


    /**
     * 管理员指定某一帐号向其它帐号发送消息
     *
     * @param fromUid     指定账号
     * @param toUid       接收账号
     * @param msgBody     消息体
     * @param sig         管理员签名
     * @param identifier  管理员账号
     * @param appId       应用id
     * @param msgLifeTime 离线保存时长
     * @param msgTypeEnum 消息类型
     * @return
     */
    public static JSONObject openimSendMsg(String fromUid, String toUid, MsgBody msgBody, String sig, String identifier,
                                           Long appId, int msgLifeTime, MsgTypeEnum msgTypeEnum, Map<String, Object> offLine) {
        //拼装json数据
        Map<Object, Object> map = new HashMap<>();
        List<Object> list = new ArrayList<>();
        //消息体内容
        list.add(setMsgBody(msgBody, msgTypeEnum));
        //1：把消息同步到From_Account在线终端和漫游上；2：消息不同步至From_Account；若不填写默认情况下会将消息存From_Account漫游
        map.put("SyncOtherMachine", 1);
        if (StringUtils.isValid(fromUid)) {
            //指定发送方账号
            map.put("From_Account", fromUid);
        }
        //消息接收方账号。
        map.put("To_Account", toUid);
        //消息离线保存时长（秒数），最长为7天（604800s）。若消息只发在线用户，不想保存离线，则该字段填0。若不填，则默认保存7天
        map.put("MsgLifeTime", msgLifeTime);
        //消息随机数,由随机函数产生。（用作消息去重）
        map.put("MsgRandom", TXCloudHelper.randomInt(Integer.MAX_VALUE));
        //消息时间戳，unix时间戳。
        map.put("MsgTimeStamp", System.currentTimeMillis() / 1000);
        //设置消息体
        map.put("MsgBody", list);
        if (!CollectionUtils.isEmpty(offLine)) {
            //离线消息设置
            map.put("OfflinePushInfo", offLine);
        }
        String json = JSON.toJSONString(map);
        String uri = TXCloudHelper.getUrl(CloudDataEnum.SENDMSG.getCode(), sig, identifier, appId);
        return JSON.parseObject(HttpClientPoolUtil.execute(uri, 0, json));
    }

    /**
     * 拉去聊天记录
     *
     * @param sig
     * @param identifier
     * @param appId
     * @return
     */
    public static List<String> getMsgSvc(String sig, String identifier, Long appId, String date) {
        //拼装json数据
        Map<Object, Object> map = new HashMap<>();
        map.put("ChatType", "C2C");
        map.put("MsgTime", date);
        String json = JSON.toJSONString(map);
        List<String> urlList = new ArrayList<>();
        String url1 = TXCloudHelper.getUrl(CloudDataEnum.GET_HISTORY.getCode(), sig, identifier, appId);
        JSONObject jObject = JSON.parseObject(HttpClientPoolUtil.execute(url1, 0, json));
        if (jObject.get("ActionStatus").toString().equals("OK")) {
            JSONArray jsonArray = jObject.getJSONArray("File");
            for (Object aJsonArray : jsonArray) {
                JSONObject o = (JSONObject) aJsonArray;
                String url = o.get("URL").toString();
                urlList.add(url);
            }
            return urlList;
        }
        return urlList;
    }

    /**
     * 离线消息设置
     *
     * @param title
     * @param desc
     * @param ext
     * @return
     */
    public static Map<String, Object> getOfflinePushInfo(String title, String desc, String ext) {
        Map<String, Object> map = new HashMap<>();
        map.put("PushFlag", 0);
        map.put("Title", title);
        map.put("Desc", desc);
        map.put("Ext", ext);
        //安卓离线消息设置
        HashMap<String, String> androidInfo = new HashMap<>();
        androidInfo.put("Sound", "android.mp3");
        map.put("AndroidInfo", androidInfo);
        //ios离线消息设置
        HashMap<String, Object> apnsInfo = new HashMap<>();
        apnsInfo.put("Sound", "apns.mp3");
        apnsInfo.put("BadgeMode", 0);
        map.put("ApnsInfo", apnsInfo);
        return map;
    }


    /**
     * 设置消息体内容
     *
     * @param msgBody
     * @param msgTypeEnum
     * @return
     */
    private static Map<String, Object> setMsgBody(MsgBody msgBody, MsgTypeEnum msgTypeEnum) {
        Map<String, Object> msgMap = new HashMap<>();
        msgMap.put("MsgType", msgTypeEnum.getCode());
        switch (msgTypeEnum) {
            case TEXT:
                HashMap<String, String> textMap = new HashMap<>(1);
                textMap.put("Text", msgBody.getText());
                msgMap.put("MsgContent", textMap);
                break;
            case LOCATION:
                HashMap<String, Object> locationMap = new HashMap<>(3);
                locationMap.put("Desc", msgBody.getDesc());
                locationMap.put("Latitude", msgBody.getLat());
                locationMap.put("Longitude", msgBody.getLng());
                msgMap.put("MsgContent", locationMap);
                break;
            case FACE:
                HashMap<String, Object> faceMap = new HashMap<>(2);
                faceMap.put("Index", msgBody.getIndex());
                faceMap.put("Data", msgBody.getData());
                msgMap.put("MsgContent", faceMap);
                break;
            case CUSTOM:
                HashMap<String, String> custMap = new HashMap<>(4);
                custMap.put("Data", msgBody.getData());
                custMap.put("Desc", msgBody.getDesc());
                custMap.put("Ext", msgBody.getExt());
                custMap.put("Sound", msgBody.getSound());
                msgMap.put("MsgContent", custMap);
                break;
            default:
                log.info("消息类型：{}", msgTypeEnum);
                throw new BusinessException(ErrorCodeEnum.ERROR_89.getCode(), ErrorCodeEnum.ERROR_89.getMsg());
        }
        return msgMap;
    }

}
